unit UfrmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, Buttons, Spin, XPMan;

type
  TfrmMain = class(TForm)
    pnlData: TPanel;
    btnCompute: TButton;
    pb: TPaintBox;
    Panel2: TPanel;
    MainMenu1: TMainMenu;
    cmFile: TMenuItem;
    cmExit: TMenuItem;
    ColorDialog1: TColorDialog;
    btnInterrupt: TBitBtn;
    Computation1: TMenuItem;
    cmCompute: TMenuItem;
    cmInterrupt: TMenuItem;
    cmResetValues: TMenuItem;
    N1: TMenuItem;
    cmOpenImage: TMenuItem;
    cmSaveImageAs: TMenuItem;
    N2: TMenuItem;
    SaveDlg: TSaveDialog;
    OpenDlg: TOpenDialog;
    rg: TRadioGroup;
    N3: TMenuItem;
    cmOpenSeriesofImages: TMenuItem;
    Timer: TTimer;
    XPManifest1: TXPManifest;
    gbData: TGroupBox;
    Label1: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Edit8: TEdit;
    SpinColors: TSpinEdit;
    gbSave: TGroupBox;
    chkProgrSave: TCheckBox;
    edtFolder: TEdit;
    btnProgrSave: TButton;
    Help1: TMenuItem;
    cmAbout: TMenuItem;
    btnPlayback: TButton;
    gbPalette: TGroupBox;
    btnReset: TSpeedButton;
    btnRandom: TSpeedButton;
    Panel3: TPanel;
    pbPal: TPaintBox;
    Label2: TLabel;
    procedure pbMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure pbMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure pbMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure pbPalPaint(Sender: TObject);
    procedure cmExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbPalMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure cmInterruptClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cmResetValuesClick(Sender: TObject);
    procedure pbPaint(Sender: TObject);
    procedure cmSaveImageAsClick(Sender: TObject);
    procedure cmOpenImageClick(Sender: TObject);
    procedure rgClick(Sender: TObject);
    procedure chkProgrSaveClick(Sender: TObject);
    procedure btnProgrSaveClick(Sender: TObject);
    procedure cmOpenSeriesofImagesClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure btnRandomClick(Sender: TObject);
    procedure cmAboutClick(Sender: TObject);
    procedure cmComputeClick(Sender: TObject);
    procedure btnPlaybackClick(Sender: TObject);
  private
      procedure ComputeColors;

      procedure ResetMandel;
      procedure ResetJulia;
  public
      backBMP  : TBitmap;
      mandBMP,
      julBMP   : TBitmap;
      Mandel   : boolean;
      DoQuit   : boolean;
      mbdown   : boolean;
      mbx,mby  : integer;
      Colors   : array[0..2048] of integer;
      ProgrSave,ProgrLoad : integer;
      GoingUp             : boolean;

  end;

const
    MAX_CONTROL_COLORS = 5;

type
    TColorArray = array[1..MAX_CONTROL_COLORS] of TColor;

const
    DefaultControlColors : TColorArray = ($200000,$FFFFFF,$FF0000,$FFFF40,$2020FF);

var
    ControlColors : TColorArray = ($200000,$FFFFFF,$FF0000,$FFFF40,$2020FF);
    MandelParams  : array[1..4] of double = (-1.5, 1.5, -2.25, 0.75);
    JuliaParams   : array[1..6] of double = (-1.8, 1.8, -1.8, 1.8, -0.56, 0.64);

var
  frmMain: TfrmMain;

implementation

uses UGlobals;

{$R *.DFM}

{ --------------------------------------------------------------------------- }
{ ---- Private Methods ------------------------------------------------------ }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.ComputeColors;
var
    i,k                                 : integer;
    r1,g1,b1,r2,g2,b2,rstep,gstep,bstep : real;

begin
    Colors[0] := 0;
    for i := 1 to MAX_CONTROL_COLORS-1 do
    begin
        b1 := (ControlColors[i] div 65536) mod 256;
        g1 := (ControlColors[i] div 256) mod 256;
        r1 := ControlColors[i] mod 256;
        b2 := (ControlColors[i+1] div 65536) mod 256;
        g2 := (ControlColors[i+1] div 256) mod 256;
        r2 := ControlColors[i+1] mod 256;
        rstep := (r2-r1)/63;
        gstep := (g2-g1)/63;
        bstep := (b2-b1)/63;
        for k := 1 to 64 do
            Colors [k+(i-1)*64] := RGB(Round(r1+rstep*(k-1)),Round(g1+gstep*(k-1)),Round(b1+bstep*(k-1)));
    end;
    for i := 257 to 2048 do
        Colors[i] := Colors[i-256];
end;

procedure TfrmMain.ResetMandel;
begin
    Edit2.Text := '100';
    Edit3.Text := '-1.5';
    Edit4.Text := '1.5';
    Edit5.Text := '-2.25';
    Edit6.Text := '0.75';
end;

procedure TfrmMain.ResetJulia;
begin
    Edit3.Text := '-1.8';
    Edit4.Text := '1.8';
    Edit5.Text := '-1.8';
    Edit6.Text := '1.8';
end;

{ --------------------------------------------------------------------------- }
{ ---- End of Private Methods ----------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Constructors, Destructors -------------------------------------------- }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.FormCreate(Sender: TObject);
begin
    Caption := AppName+' '+AppVersion+' '+AppCopyright;
    Application.Title := AppName+' '+AppVersion;
    
    ControlColors := DefaultControlColors;
    ComputeColors;
    Mandel := true;
    ProgrSave := 0;

    edtFolder.Text := StartUpDir;

    backBMP := TBitmap.Create;
    backBMP.Width := 360;
    backBMP.Height := 360;
    mandBMP := TBitmap.Create;
    mandBMP.Width := 360;
    mandBMP.Height := 360;
    julBMP := TBitmap.Create;
    julBMP.Width := 360;
    julBMP.Height := 360;

    mbdown := false;
    pb.Canvas.Pen.Color := clwhite;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    DoQuit := true;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
    cmComputeClick (nil);
end;

{ --------------------------------------------------------------------------- }
{ ---- End of Constructors, Destructors ------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Form Events ---------------------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- End of Form Events --------------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Menu Commands -------------------------------------------------------- }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.cmOpenImageClick(Sender: TObject);
var
    helpBMP : TBitmap;
begin
    if OpenDlg.Execute then
    begin
        helpBMP := TBitmap.Create;
        try
            helpBMP.LoadFromFile (OpenDlg.Filename);
            backBMP.Canvas.Draw (0,0,helpBMP);
            pb.Canvas.Draw (0,0,backBMP);
        finally
            helpBMP.Free;
        end;
    end;
end;

procedure TfrmMain.cmSaveImageAsClick(Sender: TObject);
begin
    if SaveDlg.Execute then
    begin
        backBMP.SaveToFile (SaveDlg.Filename);
    end;
end;

procedure TfrmMain.cmOpenSeriesofImagesClick(Sender: TObject);
begin
    Timer.Enabled := true;
    ProgrLoad := 0;
    GoingUp := true;
end;

procedure TfrmMain.cmExitClick(Sender: TObject);
begin
    Close;
end;

procedure TfrmMain.cmResetValuesClick(Sender: TObject);
begin
    SPinColors.Value := 256;
    case rg.ItemIndex of
    0 : ResetMandel;
    1 : ResetJulia;
    end;
    cmComputeClick(nil);
end;

procedure TfrmMain.cmComputeClick(Sender: TObject);
var
   Pmin,Pmax,Qmin,Qmax,JRe,JIm,
   x,y,x0,y0,xStep,yStep,r,p,q  : double;
   sx,sy,k,m,Kmax ,Result,limit : integer;

begin
    btnCompute.Enabled := false;
    btnInterrupt.Enabled := true;
    DoQuit := false;
    Kmax := SpinColors.Value;
    Val(Edit2.Text,M,Result);
    Val(Edit3.Text,Qmin,Result);
    Val(Edit4.Text,Qmax,Result);
    Val(Edit5.Text,Pmin,Result);
    Val(Edit6.Text,Pmax,Result);
    Val(Edit7.Text,JRe,Result);
    Val(Edit8.Text,JIm,Result);
    xStep := (Pmax-Pmin)/360;
    yStep := (Qmax-Qmin)/360;
    case Mandel of
    true  : begin
                limit := 359;
                if QMin = -QMax then
                    limit := 180;
                for sx := 0 to 359 do
                begin
                   for sy := 0 to limit do
                   begin
                       p := Pmin + xStep*sx;
                       q := Qmax - yStep*sy;
                       k := 0;
                       x0 := 0;
                       y0 := 0;
                       repeat
//                           x := x0*x0*x0*x0+y0*y0*y0*y0-6*x0*x0*y0*y0+p;
//                           y := 4*x0*y0*(x0*x0-y0*y0)+q;
//                           x := x0*x0*x0-3*x0*y0*y0+p;
//                           y := 2*x0*x0*y0+x0*x0*y0-y0*y0*y0+q;
                           x := x0*x0 - y0*y0 + p;
                           y := 2*x0*y0 + q;
                           x0 := x;
                           y0 := y;
                           Inc(k);
                           r := Sqr(x)+Sqr(y);
                       until (r>M) or (k = KMax);
                       if k = KMax then
                           k := 0;
                       backBMP.Canvas.Pixels[sx,sy] := Colors[k];
                       if (limit = 180) and (sy <> limit) and (sy <> 0) then
                           backBMP.Canvas.Pixels[sx,360-sy] := Colors[k];
                   end;
                   Application.ProcessMessages;
                   if DoQuit then
                       Exit;
            //       if (sx+1) mod 10 = 0 then
                       pb.Canvas.Draw (0,0,backBMP);
               end;
            end;
    false : begin
                limit := 359;
                if QMin = -QMax then
                    limit := 180;
                for sx := 0 to 359 do
                begin
                   for sy := 0 to limit do
                   begin
                       p := Pmin + xStep*sx;
                       q := Qmax - yStep*sy;
                       k := 0;
                       x0 := p;
                       y0 := q;
                       repeat
                           x := x0*x0 - y0*y0 + JRe;
                           y := 2*x0*y0 + JIm;
                           x0 := x;
                           y0 := y;
                           Inc(k);
                           r := Sqr(x)+Sqr(y);
                       until (r>M) or (k = KMax);
                       if k = KMax then
                           k := 0;
                       backBMP.Canvas.Pixels[sx,sy] := Colors[k];
                       if (limit = 180) and (sy <> limit) and (sy <> 0) then
                           backBMP.Canvas.Pixels[359-sx,360-sy] := Colors[k];
                   end;
                   Application.ProcessMessages;
                   if DoQuit then
                       Exit;
            //       if (sx+1) mod 10 = 0 then
                       pb.Canvas.Draw (0,0,backBMP);
               end;
            end;
    end;
    btnCompute.Enabled := true;
    btnInterrupt.Enabled := false;
    if chkProgrSave.Checked then
    begin
        backBMP.SaveToFile (edtFolder.Text+StrStringZero(ProgrSave,8,0)+'.BMP');
        Inc (ProgrSave);
    end;
    if Mandel then
    begin
        mandBMP.Canvas.Draw (0,0,backBMP);
        MandelParams[1] := QMin;
        MandelParams[2] := QMax;
        MandelParams[3] := PMin;
        MandelParams[4] := PMax;
    end
    else
    begin
        julBMP.Canvas.Draw (0,0,backBMP);
        JuliaParams[1] := QMin;
        JuliaParams[2] := QMax;
        JuliaParams[3] := PMin;
        JuliaParams[4] := PMax;
        JuliaParams[5] := JRe;
        JuliaParams[6] := JIm;
    end;
end;

procedure TfrmMain.cmInterruptClick(Sender: TObject);
begin
    DoQuit := true;
    btnInterrupt.Enabled := false;
    btnCompute.Enabled := true;
end;

procedure TfrmMain.cmAboutClick(Sender: TObject);
begin
    ShowMessage (AppName+' '+AppVersion+#13#10'written by '+AppAuthor+#13#10+AppCopyright);
end;
{ --------------------------------------------------------------------------- }
{ ---- End of Menu Commands ------------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Component Events ----------------------------------------------------- }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.rgClick(Sender: TObject);
begin
    case rg.ItemIndex of
    0 : Mandel := true;
    1 : Mandel := false;
    end;
    Edit7.Enabled := not Mandel;
    Edit8.Enabled := not Mandel;
    if Mandel then
    begin
        backBMP.Canvas.Draw (0,0,mandBMP);
        Edit3.Text := StrString (MandelParams[1],0,16);
        Edit4.Text := StrString (MandelParams[2],0,16);
        Edit5.Text := StrString (MandelParams[3],0,16);
        Edit6.Text := StrString (MandelParams[4],0,16);
    end
    else
    begin
        backBMP.Canvas.Draw (0,0,julBMP);
        Edit3.Text := StrString (JuliaParams[1],0,16);
        Edit4.Text := StrString (JuliaParams[2],0,16);
        Edit5.Text := StrString (JuliaParams[3],0,16);
        Edit6.Text := StrString (JuliaParams[4],0,16);
    end;
    pb.Canvas.Draw (0,0,backBMP);
end;

procedure TfrmMain.chkProgrSaveClick(Sender: TObject);
begin
    btnProgrSave.Enabled := chkProgrSave.Checked;
end;

procedure TfrmMain.btnProgrSaveClick(Sender: TObject);
begin
    ProgrSave := 0;
    backBMP.SaveToFile (edtFolder.Text+StrStringZero(ProgrSave,8,0)+'.BMP');
    Inc (ProgrSave);
end;

procedure TfrmMain.btnPlaybackClick(Sender: TObject);
begin
    if Timer.Enabled then
    begin
        Timer.Enabled := false;
        btnPlayback.Caption := 'Playback';
    end
    else
    begin
        Timer.Enabled := true;
        ProgrLoad := 0;
        GoingUp := true;

        btnPlayback.Caption := 'Stop Playback';
    end;
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
begin
    ControlColors := DefaultControlColors;
    ComputeColors;
    pbPal.Invalidate;
end;

procedure TfrmMain.btnRandomClick(Sender: TObject);
var
    i : integer;

begin
    for i := 1 to MAX_CONTROL_COLORS do
        ControlColors[i] := RGB(Random(256),Random(256),Random(256));
    ComputeColors;
    pbPal.Invalidate;
end;

{ --------------------------------------------------------------------------- }
{ ---- End of Component Events ---------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- PaintBox Events ------------------------------------------------------ }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.pbMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
    if Button = mbLeft then
    begin
        mbdown := true;
        mbx := x;
        mby := y;
    end;
end;

procedure TfrmMain.pbMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
var
   ny : integer;

begin
    if mbdown then
    begin
        pb.Canvas.Draw (0,0,backBMP);
//        BitBlt (pb.Canvas.Handle,0,0,360,360,backBMP.Handle,0,0,SRCCOPY);
        ny  := mby + (byte(y>mby)*2-1) * Round(360*Abs(x - mbx)/360);
        pb.Canvas.Brush.style := bsclear;
        pb.Canvas.Rectangle(mbx,mby,x,ny);
    end;
end;

procedure TfrmMain.pbMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   ny,nx,help,result   : integer;
   Pmin,Pmax,Qmin,Qmax : double;
   PW,Qw,JRe,JIm       : double;
begin
    Val(Edit3.Text,Qmin,Result);
    Val(Edit4.Text,Qmax,Result);
    Val(Edit5.Text,Pmin,Result);
    Val(Edit6.Text,Pmax,Result);

    if Button = mbRight then
    begin
        if Mandel then
        begin
            JRe := PMin+x*(Pmax-Pmin)/360;
            JIm := QMin+(360-y)*(QMax-Qmin)/360;
            Edit7.Text := StrString (JRe,0,16);
            Edit8.Text := StrString (JIm,0,16);

            rg.ItemIndex := 1;
            rgClick(nil);
            ResetJulia;
            cmComputeClick(nil);
        end;
        Exit;
    end;

    if mbdown then
    begin
        if (Abs(X-mbx) <= 2) then
        begin
            mbdown := false;
            exit;
        end;
        BitBlt (pb.Canvas.Handle,0,0,360,360,
                backBMP.Canvas.Handle,0,0,SRCCOPY);
        mbdown := false;
        nx := x;
        ny  := mby + (byte(y>mby)*2-1) * Round(360*Abs(x - mbx)/360);
        if nx < mbx then
        begin
            help := nx;
            nx := mbx;
            mbx := help;
        end;
        if ny < mby then
        begin
             help := ny;
             ny := mby;
             mby := help;
        end;
        PW := Pmax-Pmin;
        Pmin := Pmin + mbx*PW/360;
        Pmax := Pmax -(360-nx)*PW/360;
        QW := Qmax-Qmin;
        Qmin := Qmin + (360 - ny)*QW/360;
        Qmax := Qmax - mby*QW/360;
        Edit5.Text := StrString (PMin,0,16);
        Edit6.Text := StrString (PMax,0,16);
        Edit3.Text := StrString (QMin,0,16);
        Edit4.Text := StrString (QMax,0,16);
        cmComputeClick(nil);
    end;
end;

procedure TfrmMain.pbPaint(Sender: TObject);
begin
    pb.Canvas.Draw (0,0,backBMP);
end;

procedure TfrmMain.pbPalMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    i : byte;

begin
    i := Round (x/pbPal.Width*4)+1;
    ColorDialog1.Color := ControlColors[i];
    if ColorDialog1.Execute then
    begin
        ControlColors[i] := ColorDialog1.Color;
        ComputeColors;
        pbPal.Invalidate;
    end;
end;

procedure TfrmMain.pbPalPaint(Sender: TObject);
var
    i,KMax : integer;

begin
    Kmax := SpinColors.Value;
    for i := 0 to pbPal.Width do
    begin
        pbPal.Canvas.Pen.Color := Colors[Round(kMax*i/pbPal.Width)];
        pbPal.Canvas.MoveTo (i,0);
        pbPal.Canvas.LineTo (i,pbPal.Height);
    end;
end;

{ --------------------------------------------------------------------------- }
{ ---- End of PaintBox Events ----------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Protected Methods ---------------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- End of Protected Methods --------------------------------------------- }
{ --------------------------------------------------------------------------- }

{ --------------------------------------------------------------------------- }
{ ---- Timer Methods -------------------------------------------------------- }
{ --------------------------------------------------------------------------- }

procedure TfrmMain.TimerTimer(Sender: TObject);
var
    f : string;
    
begin
    f := edtFolder.Text+StrStringZero(ProgrLoad,8,0)+'.BMP';
    if FileExists (f) then
    begin
        backBMP.LoadFromFile (f);
        pb.Canvas.Draw (0,0,backBMP);
        if GoingUp then
            Inc (ProgrLoad)
        else
        begin
            Dec (ProgrLoad);
            if ProgrLoad = -1 then
            begin
                GoingUp := true;
                ProgrLoad := 0;
            end;
        end;
    end
    else
    begin
        GoingUp := false;
        Dec (ProgrLoad);
    end;
end;

{ --------------------------------------------------------------------------- }
{ ---- End of Timer Methods ------------------------------------------------- }
{ --------------------------------------------------------------------------- }


end.
